@@ -1,7 +1,7 @@
 /*
  * <security/_pam_types.h>
  *
- * $Id: _pam_types.h,v 1.10 1997/04/05 06:52:50 morgan Exp $
+ * $Id: _pam_types.h,v 1.10 1997/04/05 06:52:50 morgan Exp morgan $
  *
  * This file defines all of the types common to the Linux-PAM library
  * applications and modules.
@@ -41,80 +41,11 @@
 #include <locale.h>
 
 /* This is a blind structure; users aren't allowed to see inside a
- * pam_handle_t, so we don't define struct _pam_handle here.  This is
+ * pam_handle_t, so we don't define struct pam_handle here.  This is
  * defined in a file private to the PAM library.  (i.e., it's private
  * to PAM service modules, too!)  */
 
-typedef struct _pam_handle pam_handle_t;
-
-/* ------------ The Linux-PAM conversation structures ------------ */
-
-/* Used to pass prompting text, error messages, or other informatory
- * text to the user.  This structure is allocated and freed by the PAM
- * library (or loaded module).  */
-
-struct pam_message {
-    int msg_style;
-    const char *msg;
-};
-
-/* Used to return the user's response to the PAM library.  This
- * structure is allocated by the application program, and free()'d by
- * the Linux-PAM library (or calling module).  */
-
-struct pam_response {
-    char *resp;
-    int	resp_retcode;	/* currently un-used, zero expected */
-};
-
-/* The actual conversation structure itself */
-
-struct pam_conv {
-    int (*conv)(int num_msg, const struct pam_message **msg,
-		struct pam_response **resp, void *appdata_ptr);
-    void *appdata_ptr;
-};
-
-/* Message styles */
-
-#define PAM_PROMPT_ECHO_OFF	1
-#define PAM_PROMPT_ECHO_ON	2
-#define PAM_ERROR_MSG		3
-#define PAM_TEXT_INFO		4
-
-/* maximum size of messages */
-
-#define PAM_MAX_MSG_SIZE      200
-
-/* ---------------------- The Linux-PAM flags -------------------- */
-
-/* Authentication service should not generate any messages */
-#define PAM_SILENT			0x8000
-
-/* The authentication service should return PAM_AUTH_ERROR if the
- * user has a null authentication token */
-#define PAM_DISALLOW_NULL_AUTHTOK	0x0001
-
-/* Note: these flags are used for pam_setcred() */
-
-/* Set user credentials for an authentication service */
-#define PAM_CRED_ESTABLISH		0x0002
-
-/* Delete user credentials associated with an authentication service */
-#define PAM_CRED_DELETE			0x0004
-
-/* Reinitialize user credentials */
-#define PAM_CRED_REINITIALIZE		0x0008
-
-/* Extend lifetime of user credentials */
-#define PAM_CRED_REFRESH		0x0010
-
-/* Note: these flags are used by pam_chauthtok */
-
-/* The password service should only update those passwords that have
- * aged.  If this flag is not passed, the password service should
- * update all passwords. */
-#define PAM_CHANGE_EXPIRED_AUTHTOK	0x0020
+typedef struct pam_handle pam_handle_t;
 
 /* ----------------- The Linux-PAM return values ------------------ */
 
@@ -138,7 +69,7 @@ struct pam_conv {
 				/* maintained a retry count which has */
 				/* been reached.  No further retries */
 				/* should be attempted */
-#define PAM_AUTHTOKEN_REQD 12	/* New authentication token required. */
+#define PAM_NEW_AUTHTOK_REQD 12	/* New authentication token required. */
 				/* This is normally returned if the */
 				/* machine security policies require */
 				/* that the password should be changed */
@@ -164,8 +95,46 @@ struct pam_conv {
 				/* regardless of whether the control */
 				/* flag is required, optional, or sufficient */
 #define PAM_ABORT 26            /* Critical error (?module fail now request) */
+#define PAM_AUTHTOK_EXPIRED  27 /* user's authentication token has expired */
+#define PAM_MODULE_UNKNOWN   28 /* module is not known */
 
-#define PAM_BAD_ITEM       28   /* Bad item passed to pam_*_item() */
+/* Add new #define's here */
+
+#define PAM_BAD_ITEM       29   /* Bad item passed to pam_*_item() */
+
+/* ---------------------- The Linux-PAM flags -------------------- */
+
+/* Authentication service should not generate any messages */
+#define PAM_SILENT			0x8000U
+
+/* Note: these flags are used by pam_authenticate{,_secondary}() */
+
+/* The authentication service should return PAM_AUTH_ERROR if the
+ * user has a null authentication token */
+#define PAM_DISALLOW_NULL_AUTHTOK	0x0001U
+
+/* Note: these flags are used for pam_setcred() */
+
+/* Set user credentials for an authentication service */
+#define PAM_ESTABLISH_CRED              0x0002U
+
+/* Delete user credentials associated with an authentication service */
+#define PAM_DELETE_CRED                 0x0004U
+
+/* Reinitialize user credentials */
+#define PAM_REINITIALIZE_CRED           0x0008U
+
+/* Extend lifetime of user credentials */
+#define PAM_REFRESH_CRED                0x0010U
+
+/* Note: these flags are used by pam_chauthtok */
+
+/* The password service should only update those passwords that have
+ * aged.  If this flag is not passed, the password service should
+ * update all passwords. */
+#define PAM_CHANGE_EXPIRED_AUTHTOK	0x0020U
+
+/* ------------------ The Linux-PAM item types ------------------- */
 
 /* these defines are used by pam_set_item() and pam_get_item() */
 
@@ -180,16 +149,17 @@ struct pam_conv {
 #define PAM_RUSER       8	/* The remote user name */
 #define PAM_USER_PROMPT 9       /* the prompt for getting a username */
 
+
 /* ---------- Common Linux-PAM application/module PI ----------- */
 
 extern int pam_set_item(pam_handle_t *pamh, int item_type, const void *item);
 extern int pam_get_item(const pam_handle_t *pamh, int item_type
 			, const void **item);
-extern const char *pam_strerror(int errnum);
+extern const char *pam_strerror(pam_handle_t *pamh, int errnum);
 
 extern int pam_putenv(pam_handle_t *pamh, const char *name_value);
 extern const char * pam_getenv(pam_handle_t *pamh, const char *name);
-extern const char * const *pam_getenvlist(pam_handle_t *pamh);
+extern char **pam_getenvlist(pam_handle_t *pamh);
 
 /* ---------- Common Linux-PAM application/module PI ----------- */
 
@@ -214,6 +184,7 @@ extern const char * const *pam_getenvlist(pam_handle_t *pamh);
 #define HAVE_PAM_FAIL_DELAY
 extern int pam_fail_delay(pam_handle_t *pamh, unsigned int musec_delay);
 
+
 #ifdef MEMORY_DEBUG
 /*
  * this defines some macros that keep track of what memory has been
@@ -223,6 +194,84 @@ extern int pam_fail_delay(pam_handle_t *pamh, unsigned int musec_delay);
 #include <security/pam_malloc.h>
 #endif
 
+/* ------------ The Linux-PAM conversation structures ------------ */
+
+/* Message styles */
+
+#define PAM_PROMPT_ECHO_OFF	1
+#define PAM_PROMPT_ECHO_ON	2
+#define PAM_ERROR_MSG		3
+#define PAM_TEXT_INFO		4
+
+/* Linux-PAM specific types */
+
+#define PAM_RADIO_TYPE          5        /* yes/no/maybe conditionals */
+
+/* This is for server client non-human interaction.. these are NOT
+   part of the X/Open PAM specification (yet although Vipin has hinted
+   that they may well be 1997/7/8) but are currently included for
+   exploritory reasons.  Basically, they are for the module to obtain a
+   binary chunk of data from the client (via the server).  Such data
+   is intercepted by the server and unpacked in preparation for the
+   module */
+
+#define PAM_BINARY_MSG         6
+#define PAM_BINARY_PROMPT      7
+
+/* maximum size of messages/responses etc.. (these are mostly
+   arbitrary so Linux-PAM should handle longer values). */
+
+#define PAM_MAX_NUM_MSG       32
+#define PAM_MAX_MSG_SIZE      512
+#define PAM_MAX_RESP_SIZE     512
+
+/* Used to pass prompting text, error messages, or other informatory
+ * text to the user.  This structure is allocated and freed by the PAM
+ * library (or loaded module).  */
+
+struct pam_message {
+    int msg_style;
+    const char *msg;
+};
+
+/* if the pam_message.msg_style = PAM_BINARY_MSG or PAM_BINARY_PROMPT
+   the 'pam_message.msg' is a pointer to a 'const *' for the following
+   pseudo-structure.  When used with a PAM_BINARY_PROMPT, the returned
+   pam_response.resp pointer points to an object with the following
+   structure:
+
+   struct {
+       int length;
+       unsigned char data[length];
+   };
+
+   We define the following union to access it:
+ */
+
+union pam_u_packet_p {
+    int *length;
+    unsigned char *data;
+    char *text;
+    const char *prompt;
+};
+
+/* Used to return the user's response to the PAM library.  This
+   structure is allocated by the application program, and free()'d by
+   the Linux-PAM library (or calling module).  */
+
+struct pam_response {
+    char *resp;
+    int	resp_retcode;	/* currently un-used, zero expected */
+};
+
+/* The actual conversation structure itself */
+
+struct pam_conv {
+    int (*conv)(int num_msg, const struct pam_message **msg,
+		struct pam_response **resp, void *appdata_ptr);
+    void *appdata_ptr;
+};
+
 #ifndef LINUX_PAM
 /*
  * the following few lines represent a hack.  They are there to make
@@ -234,6 +283,7 @@ extern int pam_fail_delay(pam_handle_t *pamh, unsigned int musec_delay);
 #include <security/pam_modules.h>
 #endif
 
+
 /* ... adapted from the pam_appl.h file created by Theodore Ts'o and
  *
  * Copyright Theodore Ts'o, 1996.  All rights reserved.
