@@ -1,13 +1,13 @@
 /*
  * pam_env.c
  *
- * Copyright (c) Andrew G. Morgan <morgan@parc.power.net> 1996
+ * Copyright (c) Andrew G. Morgan <morgan@parc.power.net> 1996,1997
  * All rights reserved.
  *
  * This file was written from a "hint" provided by the people at SUN.
- * It is subject to change before reaching Linux-PAM-1.0 .
+ * and the X/Open XSSO draft of March 1997.
  *
- * $Id: pam_env.c,v 1.2 1997/02/15 15:56:48 morgan Exp $
+ * $Id: pam_env.c,v 1.2 1997/02/15 15:56:48 morgan Exp morgan $
  *
  * $Log: pam_env.c,v $
  * Revision 1.2  1997/02/15 15:56:48  morgan
@@ -15,7 +15,6 @@
  *
  * Revision 1.1  1996/12/01 03:14:13  morgan
  * Initial revision
- *
  */
 
 #include <string.h>
@@ -318,7 +317,45 @@ const char *pam_getenv(pam_handle_t *pamh, const char *name)
     }
 }
 
-const char * const *pam_getenvlist(pam_handle_t *pamh)
+static char **_copy_env(pam_handle_t *pamh)
+{
+    char **dump;
+    int i = pamh->env->requested;          /* reckon size of environment */
+    char *const *env = pamh->env->list;
+
+    D(("now get some memory for dump"));
+
+    /* allocate some memory for this (plus the null tail-pointer) */
+    dump = (char **) calloc(i, sizeof(char *));
+    D(("dump = %p", dump));
+    if (dump == NULL) {
+	return NULL;
+    }
+
+    /* now run through entries and copy the variables over */
+    dump[--i] = NULL;
+    while (i-- > 0) {
+	D(("env[%d]=`%s'", i,env[i]));
+	dump[i] = _pam_strdup(env[i]);
+	D(("->dump[%d]=`%s'", i,dump[i]));
+	if (dump[i] == NULL) {
+	    /* out of memory */
+
+	    while (dump[++i]) {
+		_pam_overwrite(dump[i]);
+		_pam_drop(dump[i]);
+	    }
+	    return NULL;
+	}
+    }
+
+    env = NULL;                             /* forget now */
+
+    /* return transcribed environment */
+    return dump;
+}
+
+char **pam_getenvlist(pam_handle_t *pamh)
 {
     int i;
 
@@ -347,9 +384,9 @@ const char * const *pam_getenvlist(pam_handle_t *pamh)
 	}
     }
 
-    /* Seems fine; return environment */
+    /* Seems fine; copy environment */
 
     _pam_dump_env(pamh);                    /* only active when debugging */
 
-    return ((const char * const *) pamh->env->list);
+    return _copy_env(pamh);
 }
