@@ -4,7 +4,7 @@
  * created by Marc Ewing.
  * Currently maintained by Andrew G. Morgan <morgan@parc.power.net>
  *
- * $Id: pam_handlers.c,v 1.17 1997/04/05 06:55:24 morgan Exp $
+ * $Id: pam_handlers.c,v 1.17 1997/04/05 06:55:24 morgan Exp morgan $
  *
  */
 
@@ -13,12 +13,26 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dlfcn.h>
+#ifdef PAM_SHL
+# include <dl.h>
+#else
+# include <dlfcn.h>
+#endif
 #include <fcntl.h>
 #include <unistd.h>
 
 #include "pam_private.h"
 
+/* FreeBSD doesn't define this */
+#ifndef RTLD_NOW
+# define RTLD_NOW      1
+#endif
+
+/* If not required, define as nothing - FreeBSD needs it to be "_"... */
+#ifndef SHLIB_SYM_PREFIX
+# define SHLIB_SYM_PREFIX ""
+#endif
+
 #define BUF_SIZE      1024
 #define MODULE_CHUNK  4
 
@@ -267,7 +281,7 @@ int _pam_init_handlers(pam_handle_t *pamh)
 		    _pam_log_error("_pam_init_handlers: error reading %s"
 				   , filename);
 		    _pam_log_error("_pam_init_handlers: [%s]"
-				   , pam_strerror(retval));
+				   , pam_strerror(pamh, retval));
 		} else {
 		    read_something = 1;
 		}
@@ -307,7 +321,7 @@ int _pam_init_handlers(pam_handle_t *pamh)
 			_pam_log_error("_pam_init_handlers: error reading %s"
 				       , PAM_DEFAULT_SERVICE_FILE);
 			_pam_log_error("_pam_init_handlers: [%s]"
-				       , pam_strerror(retval));
+				       , pam_strerror(pamh, retval));
 		    } else {
 			read_something = 1;
 		    }
@@ -446,6 +460,9 @@ int _pam_add_handler(pam_handle_t *pamh
     struct handler **handler_p2;
     struct handlers *the_handlers;
     const char *sym, *sym2;
+#ifdef PAM_SHL
+    const char *_sym, *_sym2;
+#endif
     char *mod_full_path=NULL;
     servicefn func, func2;
     int success;
@@ -494,12 +511,19 @@ int _pam_add_handler(pam_handle_t *pamh
 
 #ifdef PAM_DYNAMIC
 	D(("_pam_add_handler: dlopen(%s) -> %lx", mod_path, &mod->dl_handle));
-	mod->dl_handle = dlopen(mod_path, RTLD_NOW);
+	mod->dl_handle =
+# ifdef PAM_SHL
+	    shl_load(mod_path, BIND_IMMEDIATE, 0L);
+# else /* PAM_SHL */
+	    dlopen(mod_path, RTLD_NOW);
+# endif /* PAM_SHL */
 	D(("_pam_add_handler: dlopen'ed"));
 	if (mod->dl_handle == NULL) {
 	    D(("_pam_add_handler: dlopen(%s) failed", mod_path));
 	    _pam_log_error("unable to dlopen(%s)", mod_path);
+# ifndef PAM_SHL
 	    _pam_log_error("[dlerror: %s]", dlerror());
+# endif /* PAM_SHL */
 	    /* Don't abort yet; static code may be able to find function.
 	     * But defaults to abort if nothing found below... */
 	} else {
@@ -569,29 +593,46 @@ int _pam_add_handler(pam_handle_t *pamh
 
     handler_p = handler_p2 = NULL;
     func = func2 = NULL;
+#ifdef PAM_SHL
+    _sym2 =
+#endif /* PAM_SHL */
     sym2 = NULL;
 
     /* point handler_p's at the root addresses of the function stacks */
     switch (type) {
     case PAM_T_AUTH:
 	handler_p = &the_handlers->authenticate;
-	sym = "pam_sm_authenticate";
+	sym = SHLIB_SYM_PREFIX "pam_sm_authenticate";
 	handler_p2 = &the_handlers->setcred;
-	sym2 = "pam_sm_setcred";
+	sym2 = SHLIB_SYM_PREFIX "pam_sm_setcred";
+#ifdef PAM_SHL
+	_sym = "_pam_sm_authenticate";
+	_sym2 = "_pam_sm_setcred";
+#endif
 	break;
     case PAM_T_SESS:
 	handler_p = &the_handlers->open_session;
-	sym = "pam_sm_open_session";
+	sym = SHLIB_SYM_PREFIX "pam_sm_open_session";
 	handler_p2 = &the_handlers->close_session;
-	sym2 = "pam_sm_close_session";
+	sym2 = SHLIB_SYM_PREFIX "pam_sm_close_session";
+#ifdef PAM_SHL
+	_sym = "_pam_sm_open_session";
+	_sym2 = "_pam_sm_close_session";
+#endif
 	break;
     case PAM_T_ACCT:
 	handler_p = &the_handlers->acct_mgmt;
-	sym = "pam_sm_acct_mgmt";
+	sym = SHLIB_SYM_PREFIX "pam_sm_acct_mgmt";
+#ifdef PAM_SHL
+	_sym = "_pam_sm_acct_mgmt";
+#endif
 	break;
     case PAM_T_PASS:
 	handler_p = &the_handlers->chauthtok;
-	sym = "pam_sm_chauthtok";
+	sym = SHLIB_SYM_PREFIX "pam_sm_chauthtok";
+#ifdef PAM_SHL
+	_sym = "_pam_sm_chauthtok";
+#endif
 	break;
     default:
 	/* Illegal module type */
@@ -612,7 +653,7 @@ int _pam_add_handler(pam_handle_t *pamh
 	 mod->type != PAM_MT_FAULTY_MOD
 	) {
 	D(("_pam_add_handlers: illegal module library type; %d", mod->type));
-	_pam_log_error("internal error: module libraray type not known: %s;%d"
+	_pam_log_error("internal error: module library type not known: %s;%d"
 		       , sym, mod->type);
 	return PAM_ABORT;
     }
@@ -621,7 +662,13 @@ int _pam_add_handler(pam_handle_t *pamh
     
 #ifdef PAM_DYNAMIC
     if ((mod->type == PAM_MT_DYNAMIC_MOD) &&
-        (func = (servicefn) dlsym(mod->dl_handle, sym)) == NULL) {
+# ifdef PAM_SHL
+	(shl_findsym(&mod->dl_handle, sym, (short) TYPE_PROCEDURE, &func) &&
+	 shl_findsym(&mod->dl_handle, _sym, (short) TYPE_PROCEDURE, &func))
+# else /* PAM_SHL */
+        (func = (servicefn) dlsym(mod->dl_handle, sym)) == NULL
+# endif /* PAM_SHL */
+	) {
 	_pam_log_error("unable to resolve symbol: %s", sym);
     }
 #endif
@@ -634,7 +681,13 @@ int _pam_add_handler(pam_handle_t *pamh
     if (sym2) {
 #ifdef PAM_DYNAMIC
 	if ((mod->type == PAM_MT_DYNAMIC_MOD) &&
-	    (func2 = (servicefn) dlsym(mod->dl_handle, sym2)) == NULL) {
+# ifdef PAM_SHL
+	    (shl_findsym(&mod->dl_handle,sym2,(short)TYPE_PROCEDURE, &func2)&&
+	     shl_findsym(&mod->dl_handle,_sym2,(short)TYPE_PROCEDURE, &func2))
+# else /* PAM_SHL */
+	    (func2 = (servicefn) dlsym(mod->dl_handle, sym2)) == NULL
+# endif /* PAM_SHL */
+	    ) {
 	    _pam_log_error("unable to resolve symbol: %s", sym2);
 	}
 #endif
@@ -715,7 +768,11 @@ int _pam_free_handlers(pam_handle_t *pamh)
 	D(("_pam_free_handlers: dlclose(%s)", mod->name));
 	free(mod->name);
 #ifdef PAM_DYNAMIC
+# ifdef PAM_SHL
+	if (mod->type == PAM_MT_DYNAMIC_MOD) shl_unload(mod->dl_handle);
+# else
 	if (mod->type == PAM_MT_DYNAMIC_MOD) dlclose(mod->dl_handle);
+# endif
 #endif
 	mod++;
 	pamh->handlers.modules_used--;
