@@ -1,7 +1,7 @@
 /*
  * pam_private.h
  *
- * $Id: pam_private.h,v 1.12 1997/04/05 06:57:37 morgan Exp $
+ * $Id: pam_private.h,v 1.12 1997/04/05 06:57:37 morgan Exp morgan $
  *
  * This is the Linux-PAM Library Private Header. It contains things
  * internal to the Linux-PAM library. Things not needed by either an
@@ -110,11 +110,6 @@ struct pam_environ {
 				  * of pointers to malloc() memory)  */
 };
 
-#ifdef PAM_FAIL_DELAY_ON
-
-/* warning this code is likely to change. It was added for .50 and is
-   a rather crude attempt to provide some time delay support */
-
 #include <sys/time.h>
 
 typedef enum { PAM_FALSE, PAM_TRUE } _pam_boolean;
@@ -125,9 +120,7 @@ struct _pam_fail_delay {
      time_t begin;
 };
 
-#endif /* PAM_FAIL_DELAY_ON */
-
-struct _pam_handle {
+struct pam_handle {
     char *service_name;
     char *user;
     char *prompt;                /* for use by pam_get_user() -- AGM */
@@ -139,9 +132,7 @@ struct _pam_handle {
     struct pam_conv *pam_conversation;
     struct pam_data *data;
     struct pam_environ *env;      /* structure to maintain environment list */
-#ifdef PAM_FAIL_DELAY_ON
     struct _pam_fail_delay fail_delay;   /* helper function for easy delays */
-#endif /* PAM_FAIL_DELAY_ON */
     struct service handlers;
 };
 
