@@ -1,5 +1,5 @@
 /*
- * $Id: misc_conv.c,v 1.5 1997/01/04 20:16:48 morgan Exp $
+ * $Id: misc_conv.c,v 1.5 1997/01/04 20:16:48 morgan Exp morgan $
  *
  * A generic conversation function for text based applications
  *
@@ -23,6 +23,9 @@
  * Initial revision
  */
 
+#define _GNU_SOURCE
+#include <features.h>
+
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,7 +39,6 @@
 #include <security/pam_misc.h>
 
 #define INPUTSIZE PAM_MAX_MSG_SIZE           /* maximum length of input+1 */
-#define REPLY_CHUNK 5               /* number of replies to get at a time */
 #define CONV_ECHO_ON  1                            /* types of echo state */
 #define CONV_ECHO_OFF 0
 
@@ -53,6 +55,21 @@ const char *pam_misc_conv_die_line  = "..\a.Sorry, your time is up!\n";
 
 int pam_misc_conv_died=0;       /* application can probe this for timeout */
 
+/* This function pointer is set to NULL if the function is not
+   availiable.  One or both of the arguments to this function must be
+   non-NULL.  This function must return PAM_SUCCESS or PAM_CONV_ERR.
+   If input is non-NULL, a response is expected, this response should
+   be malloc()'d and will eventually be free()'d by the calling
+   module. The structure of this malloc()'d response is as follows:
+
+          { int length, char data[length] }
+
+   The union is a _pointer_ to either an int or a char and can thus be
+   conveniently addressed to fill in the memory it points to. */
+
+int (*pam_binary_handler_fn)(const union pam_u_packet_p send,
+			     union pam_u_packet_p *receive) = NULL;
+
 /* the following code is used to get text input */
 
 volatile static int expired=0;
@@ -132,7 +149,7 @@ static char *read_string(int echo, const char *prompt)
     struct sigaction old_sig;
     int delay, nc, have_term=0;
 
-    D(("called."));
+    D(("called with echo='%s', prompt='%s'.", echo ? "ON":"OFF" , prompt));
 
     if (isatty(STDIN_FILENO)) {                      /* terminal state */
 
@@ -177,16 +194,22 @@ static char *read_string(int echo, const char *prompt)
 	    }
 	    if (expired) {
 		delay = get_delay();
-	    } else if (nc >= 0) {                /* we got some user input */
+	    } else if (nc > 0) {                 /* we got some user input */
 		char *input;
+
 		if (nc > 0 && line[nc-1] == '\n') {     /* <NUL> terminate */
 		    line[--nc] = '\0';
 		} else {
 		    line[nc] = '\0';
 		}
-		input = xstrdup(line);
+		input = x_strdup(line);
 		_pam_overwrite(line);
+
 		return input;                  /* return malloc()ed string */
+	    } else if (nc == 0) {                                /* Ctrl-D */
+		D(("user did not want to type anything"));
+		fprintf(stderr, "\n");
+		break;
 	    }
 	}
     }
@@ -201,88 +224,103 @@ static char *read_string(int echo, const char *prompt)
 
 /* end of read_string functions */
 
-static void drop_reply(struct pam_response *reply, int replies)
-{
-     int i;
-
-     for (i=0; i<replies; ++i) {
-	  _pam_overwrite(reply[i].resp);      /* might be a password */
-	  free(reply[i].resp);
-     }
-     if (reply)
-	  free(reply);
-}
-
 int misc_conv(int num_msg, const struct pam_message **msgm,
-		     struct pam_response **response, void *appdata_ptr)
+	      struct pam_response **response, void *appdata_ptr)
 {
-     int count=0,replies=0,space=0;
-     struct pam_response *reply=NULL;
-     char *string=NULL;
+    int count=0;
+    struct pam_response *reply;
 
-     D(("entering conversation function."));
+    if (num_msg <= 0)
+	return PAM_CONV_ERR;
 
-     for (count=0; count < num_msg; ++count) {
-	  switch (msgm[count]->msg_style) {
-	  case PAM_PROMPT_ECHO_OFF:
-	       string = read_string(CONV_ECHO_OFF,msgm[count]->msg);
-	       if (string == NULL) {
-		    drop_reply(reply,replies);
-		    return (PAM_CONV_ERR);
-	       }
-	       break;
-	  case PAM_PROMPT_ECHO_ON:
-	       string = read_string(CONV_ECHO_ON,msgm[count]->msg);
-	       if (string == NULL) {
-		    drop_reply(reply,replies);
-		    return (PAM_CONV_ERR);
-	       }
-	       break;
-	  case PAM_ERROR_MSG:
-	       fprintf(stderr,"%s\n",msgm[count]->msg);
-	       break;
-	  case PAM_TEXT_INFO:
-	       fprintf(stderr,"%s\n",msgm[count]->msg);
-	       break;
-	  default:
-	       fprintf(stderr, "erroneous conversation (%d)\n"
-		       ,msgm[count]->msg_style);
-	       drop_reply(reply,replies);
-	       return (PAM_CONV_ERR);
-	  }
+    D(("allocating empty response structure array."));
 
-	  if (string) {     /* must add to reply array */
-	       struct pam_response *ptmp;
+    reply = (struct pam_response *) calloc(num_msg,
+					   sizeof(struct pam_response));
+    if (reply == NULL) {
+	D(("no memory for responses"));
+	return PAM_CONV_ERR;
+    }
 
-	       /* do we need a larger reply array ? */
+    D(("entering conversation function."));
 
-	       if (space <= replies) {
-		    space += REPLY_CHUNK;
-		    ptmp = (struct pam_response *)
-			 realloc(reply, space*sizeof(struct pam_response));
-		    if (ptmp == NULL) {
-			 drop_reply(reply,replies);
-			 return PAM_CONV_ERR;        /* ran out of memory */
-		    }
-		    reply = ptmp;                       /* enlarged array */
-	       }
+    for (count=0; count < num_msg; ++count) {
+	char *string=NULL;
 
-	       /* add string to list of responses */
+	switch (msgm[count]->msg_style) {
+	case PAM_PROMPT_ECHO_OFF:
+	    string = read_string(CONV_ECHO_OFF,msgm[count]->msg);
+	    if (string == NULL) {
+		goto failed_conversation;
+	    }
+	    break;
+	case PAM_PROMPT_ECHO_ON:
+	    string = read_string(CONV_ECHO_ON,msgm[count]->msg);
+	    if (string == NULL) {
+		goto failed_conversation;
+	    }
+	    break;
+	case PAM_ERROR_MSG:
+	    if (fprintf(stderr,"%s\n",msgm[count]->msg) < 0) {
+		goto failed_conversation;
+	    }
+	    break;
+	case PAM_TEXT_INFO:
+	    if (fprintf(stdout,"%s\n",msgm[count]->msg) < 0) {
+		goto failed_conversation;
+	    }
+	    break;
+	case PAM_BINARY_PROMPT:
+	{
+	    union pam_u_packet_p pack_in, pack_out;
 
-	       reply[replies].resp_retcode = 0;
-	       reply[replies++].resp = string;
-	       string = NULL;
-	  }
-     }
+	    pack_in.prompt = msgm[count]->msg;
+	    pack_out.data = NULL;
 
-     /* do we need to bother with a response? */
-     if (reply) {
+	    if (!pam_binary_handler_fn
+		|| pam_binary_handler_fn(pack_in,&pack_out)
+		!= PAM_SUCCESS
+		|| string == NULL) {
+		goto failed_conversation;
+	    }
+	    string = pack_out.text;
 
-	  /* note, this pam_response structure (array) will be
-	   * free()'d by the module */
+	    break;
+	}
+	case PAM_BINARY_MSG:
+	{
+	    union pam_u_packet_p pack_in;
 
-	  *response = reply;
-     }
+	    pack_in.prompt = msgm[count]->msg;
+	    if (!pam_binary_handler_fn
+		|| pam_binary_handler_fn(pack_in,(union pam_u_packet_p *) NULL)
+		!= PAM_SUCCESS) {
+		goto failed_conversation;
+	    }
+	    break;
+	}
+	default:
+	    fprintf(stderr, "erroneous conversation (%d)\n"
+		    ,msgm[count]->msg_style);
+	    goto failed_conversation;
+	}
+
+	if (string) {                         /* must add to reply array */
+	    /* add string to list of responses */
+
+	    reply[count].resp_retcode = 0;
+	    reply[count].resp = string;
+	}
+    }
+
+    /* New (0.59+) behavior is to always have a reply - this is
+       compatable with the X/Open (March 1997) spec. */
+    *response = reply;
      
-     return PAM_SUCCESS;
+    return PAM_SUCCESS;
+
+failed_conversation:
+
+    _pam_drop_reply(reply, num_msg);
+    return PAM_CONV_ERR;
 }
