@@ -1,7 +1,7 @@
 /* pam_ftp module */
 
 /*
- * $Id: pam_ftp.c,v 1.2 1997/02/15 16:23:59 morgan Exp $
+ * $Id: pam_ftp.c,v 1.2 1997/02/15 16:23:59 morgan Exp morgan $
  *
  * Written by Andrew Morgan <morgan@parc.power.net> 1996/3/11
  *
@@ -22,6 +22,9 @@
 /* the following is a password that "can't be correct" */
 #define BLOCK_PASSWORD "\177BAD PASSWPRD\177"
 			    
+#define _GNU_SOURCE
+#include <features.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -43,19 +46,6 @@
 
 /* some syslogging */
 
-static char *xstrdup(const char *x)
-{
-    char *y;
-
-    if (x) {
-	y = (char *) malloc(strlen(x)+1);
-	if (y)
-	    strcpy(y,x);
-	return y;
-    } else
-	return NULL;
-}
-
 static void _pam_log(int err, const char *format, ...)
 {
     va_list args;
@@ -86,12 +76,12 @@ static int converse(pam_handle_t *pamh, int nargs
 
 	if (retval != PAM_SUCCESS) {
 	    _pam_log(LOG_DEBUG, "conversation failure [%s]"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh, retval));
 	}
 
     } else {
 	_pam_log(LOG_ERR, "couldn't obtain coversation function [%s]"
-		 , pam_strerror(retval));
+		 , pam_strerror(pamh, retval));
     }
 
     D(("ready to return from module conversation\n"));
@@ -117,7 +107,7 @@ static int _pam_parse(int argc, const char **argv, char **users)
 	if (!strcmp(*argv,"debug"))
 	    ctrl |= PAM_DEBUG_ARG;
 	else if (!strncmp(*argv,"users=",6)) {
-	    *users = xstrdup(6+*argv);
+	    *users = x_strdup(6+*argv);
 	    if (*users == NULL) {
 		ctrl |= PAM_NO_ANON;
 		_pam_log(LOG_CRIT, "failed to duplicate user list - anon off");
@@ -229,12 +219,13 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 	    msg[i].msg = GUEST_LOGIN_PROMPT;
 	}
 
-	retval = converse(pamh, i, mesg, &resp);
+	retval = converse(pamh, ++i, mesg, &resp);
 	_pam_overwrite(prompt);
 	_pam_drop(prompt);
 
-	if (retval != PAM_SUCCESS || resp == NULL || resp->resp == NULL) {
-	    _pam_drop(resp);
+	if (retval != PAM_SUCCESS) {
+	    if (resp != NULL)
+		_pam_drop_reply(resp,i);
 	    return PAM_AUTHINFO_UNAVAIL;
 	}
 
@@ -266,9 +257,7 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 	}
 
 	if (resp) {                                      /* clean up */
-	    _pam_overwrite(resp->resp);
-	    _pam_drop(resp->resp);
-	    _pam_drop(resp);
+	    _pam_drop_reply(resp, i);
 	}
 
 	/* success or failure */
