@@ -1,7 +1,7 @@
 /* pam_mail module */
 
 /*
- * $Id: pam_mail.c,v 1.2 1997/02/15 16:06:14 morgan Exp $
+ * $Id: pam_mail.c,v 1.2 1997/02/15 16:06:14 morgan Exp morgan $
  *
  * Written by Andrew Morgan <morgan@parc.power.net> 1996/3/11
  * $HOME additions by David Kinchlea <kinch@kinch.ark.com> 1997/1/7
@@ -20,6 +20,9 @@
 #define MAIL_ENV_FORMAT           MAIL_ENV_NAME "=%s"
 #define YOUR_MAIL_FORMAT          "You have %s mail in %s"
 
+#define _GNU_SOURCE
+#include <features.h>
+
 #include <ctype.h>
 #include <pwd.h>
 #include <stdarg.h>
@@ -49,19 +52,6 @@
 
 /* some syslogging */
 
-static char *xstrdup(const char *x)
-{
-    if (x) {
-	char *y;
-
-	y = (char *) malloc(strlen(x)+1);
-	if (y)
-	    strcpy(y,x);
-	return y;
-    } else
-	return NULL;
-}
-
 static void _log_err(int err, const char *format, ...)
 {
     va_list args;
@@ -100,7 +90,7 @@ static int _pam_parse(int flags, int argc, const char **argv, char **maildir)
 	if (!strcmp(*argv,"debug"))
 	    ctrl |= PAM_DEBUG_ARG;
 	else if (!strncmp(*argv,"dir=",4)) {
-	    *maildir = xstrdup(4+*argv);
+	    *maildir = x_strdup(4+*argv);
 	    if (*maildir != NULL) {
 		D(("new mail directory: %s", *maildir));
 		ctrl |= PAM_NEW_MAIL_DIR;
@@ -145,12 +135,12 @@ static int converse(pam_handle_t *pamh, int ctrl, int nargs
 
 	if (retval != PAM_SUCCESS && (PAM_DEBUG_ARG & ctrl) ) {
 	    _log_err(LOG_DEBUG, "conversation failure [%s]"
-		     , pam_strerror(retval));
+		     , pam_strerror(pamh, retval));
 	}
 
     } else {
 	_log_err(LOG_ERR, "couldn't obtain coversation function [%s]"
-		 , pam_strerror(retval));
+		 , pam_strerror(pamh, retval));
     }
 
     D(("ready to return from module conversation"));
@@ -275,13 +265,15 @@ static int report_mail(pam_handle_t *pamh, int ctrl
 
 	    _pam_overwrite(remark);
 	    _pam_drop(remark);
+	    if (resp)
+		_pam_drop_reply(resp, 1);
 	}
     } else {
 	D(("keeping quiet"));
 	retval = PAM_SUCCESS;
     }
 
-    D(("returning %s", pam_strerror(retval)));
+    D(("returning %s", pam_strerror(pamh, retval)));
     return retval;
 }
 
@@ -319,8 +311,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
 
     /* Do we have anything to do? */
 
-    flags &= ~PAM_SILENT;
-    if (flags != PAM_CRED_ESTABLISH && flags != PAM_CRED_DELETE) {
+    if (!(flags & (PAM_ESTABLISH_CRED|PAM_DELETE_CRED))) {
 	return PAM_SUCCESS;
     }
 
@@ -334,7 +325,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
 
     /* set the MAIL variable? */
 
-    if (!(ctrl & PAM_NO_ENV) && (flags & PAM_CRED_ESTABLISH)) {
+    if (!(ctrl & PAM_NO_ENV) && (flags & PAM_ESTABLISH_CRED)) {
 	char *tmp;
 
 	tmp = malloc(strlen(folder)+sizeof(MAIL_ENV_FORMAT));
@@ -364,8 +355,8 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
      * OK. we've got the mail folder... what about its status?
      */
 
-    if ((flags == PAM_CRED_ESTABLISH && !(ctrl & PAM_NO_LOGIN))
-	|| (flags == PAM_CRED_DELETE && ctrl & PAM_LOGOUT_TOO)) {
+    if (((flags & PAM_ESTABLISH_CRED) && !(ctrl & PAM_NO_LOGIN))
+	|| ((flags & PAM_DELETE_CRED) && (ctrl & PAM_LOGOUT_TOO))) {
 	type = get_mail_status(ctrl, folder);
 	if (type != NULL) {
 	    retval = report_mail(pamh, ctrl, type, folder);
@@ -377,7 +368,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
      * Delete environment variable?
      */
 
-    if (flags == PAM_CRED_DELETE) {
+    if (flags & PAM_DELETE_CRED) {
 	(void) pam_putenv(pamh, MAIL_ENV_NAME);
     }
 
