@@ -3,6 +3,8 @@
  * Modifications, Cristian Gafton 97/2/8
  * Modifications, Peter Allgeyer 97/3
  * Modifications (netgroups and fixes), Nicolai Langfeldt 97/3/21
+ * Security fix: 97/10/2 - gethostbyname called repeatedly without care
+ * Modification (added privategroup option) Andrew <morgan@transmeta.com>
  *----------------------------------------------------------------------
  * Copyright (c) 1983, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -36,6 +38,8 @@
  * SUCH DAMAGE.
  */
 
+#define _BSD_SOURCE
+
 #define USER_RHOSTS_FILE "/.rhosts"     /* prefixed by user's home dir */
 
 #ifdef linux
@@ -45,7 +49,6 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <string.h>
-#define __USE_BSD
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/param.h>
@@ -56,12 +59,9 @@ int innetgr(const char *, const char *, const char *,const char *);
 
 #include <stdio.h>
 #include <errno.h>
-#include <string.h>
-#include <unistd.h>
 #include <sys/time.h>
 #include <arpa/inet.h>
 
-#define __USE_BSD
 #ifndef MAXDNAME
 #define MAXDNAME  256
 #endif
@@ -74,12 +74,19 @@ int innetgr(const char *, const char *, const char *,const char *);
 #include <linux/sockios.h>
 #endif
 
+#ifndef __USE_MISC
 #define __USE_MISC
+#include <sys/fsuid.h>
+#endif /* __USE_MISC */
 #include <pwd.h>
+#include <grp.h>
 #include <sys/file.h>
 #include <sys/signal.h>
 #include <sys/stat.h>
 #include <sys/syslog.h>
+#ifndef _PATH_HEQUIV
+#define _PATH_HEQUIV "/etc/hosts.equiv"
+#endif /* _PATH_HEQUIV */
 
 #define PAM_SM_AUTH  /* only defines this management group */
 
@@ -99,6 +106,7 @@ struct _options {
     int  opt_silent;
     int  opt_promiscuous;
     int  opt_suppress;
+    int  opt_private_group;
     const char *last_error;
 };
 
@@ -116,37 +124,43 @@ static void _pam_log(int err, const char *format, ...)
 
 static void set_option (struct _options *opts, const char *arg)
 {
-    if (strcmp (arg, "no_hosts_equiv") == 0) {
+    if (strcmp(arg, "no_hosts_equiv") == 0) {
 	opts->opt_no_hosts_equiv = 1;
 	return;
     }
 
-    if (strcmp (arg, "no_rhosts") == 0) {
+    if (strcmp(arg, "no_rhosts") == 0) {
 	opts->opt_no_rhosts = 1;
 	return;
     }
 
-    if (strcmp (arg, "debug") == 0) {
+    if (strcmp(arg, "debug") == 0) {
 	D(("debugging enabled"));
 	opts->opt_debug = 1;
 	return;
     }
 
-    if (strcmp (arg, "no_warn") == 0) {
+    if (strcmp(arg, "no_warn") == 0) {
 	opts->opt_nowarn = 1;
 	return;
     }
 
-    if (strcmp(arg, "promiscuous")) {
+    if (strcmp(arg, "promiscuous") == 0) {
 	opts->opt_promiscuous = 1;   /* used to permit '+' in ...hosts file */
 	return;
     }
 
-    if (strcmp(arg, "suppress")) {
+    if (strcmp(arg, "suppress") == 0) {
 	opts->opt_suppress = 1; /* used to suppress failure warning message */
 	return;
     }
 
+    if (strcmp(arg, "privategroup") == 0) {
+	opts->opt_private_group = 1; /* used to permit group write on .rhosts
+					file if group has same name as owner */
+	return;
+    }
+
     /*
      * All other options are ignored at the present time.
      */
@@ -301,6 +315,19 @@ static int __icheckuser(pam_handle_t *pamh, struct _options *opts
     return (strcmp(ruser, luser) == 0);
 }
 
+/*
+ * Returns 1 for blank lines (or only comment lines) and 0 otherwise
+ */
+
+static int __isempty(char *p)
+{
+    while (*p && isspace(*p)) {
+	++p;
+    }
+
+    return (*p == '\0' || *p == '#') ? 1:0 ;
+}
+
 /*
  * Returns 0 if positive match, 1 if _not_ ok.
  */
@@ -320,6 +347,11 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
     while (fgets(buf, sizeof (buf)-1, hostf)) {   /* line from hostf file */
         p = buf;                              /* from beginning of file.. */
 
+	/* Skip empty or comment lines */
+	if (__isempty(p)) {
+	    continue;
+	}
+
 	/* Skip lines that are too long. */
 	if (strchr(p, '\n') == (char *) 0) {
 	    ch = getc(hostf);
@@ -396,7 +428,7 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
  */
 
 static int
-iruserok(pam_handle_t *pamh,
+pam_iruserok(pam_handle_t *pamh,
 	 struct _options *opts, u_long raddr, int superuser,
 	 const char *ruser, const char *luser, const char *rhost)
 {
@@ -452,7 +484,7 @@ iruserok(pam_handle_t *pamh,
 
     /*
      * Change effective uid while _reading_ .rhosts. (not just
-     * opening.  If root and reading an NFS mounted file system,
+     * opening).  If root and reading an NFS mounted file system,
      * can't read files that are 0600 as .rhosts files should be.
      */
 
@@ -482,14 +514,42 @@ iruserok(pam_handle_t *pamh,
     cp = NULL;
     if (lstat(pbuf, &sbuf) < 0 || !S_ISREG(sbuf.st_mode))
 	cp = ".rhosts not regular file";
-
-    if (fstat(fileno(hostf), &sbuf) < 0)
+    else if (fstat(fileno(hostf), &sbuf) < 0)
 	cp = ".rhosts fstat failed";
     else if (sbuf.st_uid && sbuf.st_uid != pwd->pw_uid)
 	cp = "bad .rhosts owner";
-    else if (sbuf.st_mode & (S_IWGRP|S_IWOTH))
-	cp = ".rhosts writable by other than owner";
-    
+    else if (sbuf.st_mode & S_IWOTH)
+	cp = ".rhosts writable by other!";
+    else if (sbuf.st_mode & S_IWGRP) {
+
+	/* private group caveat */
+	if (opts->opt_private_group) {
+	    struct group *grp = getgrgid(sbuf.st_gid);
+
+	    if (NULL == grp || NULL == grp->gr_name
+		|| strcmp(luser,grp->gr_name)) {
+		cp = ".rhosts writable by public group";
+	    } else if (grp->gr_mem) {
+		int gcount;
+
+		/* require at most one member (luser) of this group */
+		for (gcount=0; grp->gr_mem[gcount]; ++gcount) {
+		    if (strcmp(grp->gr_mem[gcount], luser)) {
+			gcount = -1;
+			break;
+		    }
+		}
+		if (gcount < 0) {
+		    cp = ".rhosts writable by other members of group";
+		}
+	    }
+	} else {
+	    cp = ".rhosts writable by group";
+	}
+
+    } /* It is _NOT_ safe to append an else here...  Do so prior to
+       * S_IWGRP check */
+
     /* If there were any problems, quit. */
     if (cp) {
 	opts->last_error = cp;
@@ -524,25 +584,31 @@ pam_ruserok (pam_handle_t *pamh,
 {
     struct hostent *hp;
     int answer = 1;                             /* default to failure */
-    u_long addr;
-    char **ap;
+    u_long *addrs;
+    int n, i;
 
     opts->last_error = (char *) 0;
     hp               = gethostbyname(rhost);         /* identify host */
 
     if (hp != NULL) {
-	ap = hp->h_addr_list;
-	while (*ap) {                     /* loop though address list */
-	    memcpy (&addr, *ap, sizeof(addr));
+        /* loop though address list */
+        for (n = 0; hp->h_addr_list[n]; n++);
+        D(("rhosts: %d addresses", n))
 
-	    /* check user on remote host */
-	    if (iruserok(pamh, opts, addr, superuser, ruser, luser, rhost)
-		== 0) {
-		answer = 0;                                /* success */
-		break;
-	    }
-	    ++ap;
-	}
+        if (n) {
+            addrs = calloc (n, hp->h_length);
+            for (i = 0; i < n; i++)
+                memcpy (addrs+i, hp->h_addr_list[i], hp->h_length);
+
+            for (i = 0; i < n && answer; i++) {
+                D(("rhosts: address %d is %04x", i, addrs[i]))
+                answer = pam_iruserok(pamh, opts, addrs[i], superuser,
+				      ruser, luser, rhost);
+                         /* answer == 0 means success */
+            }
+
+            free (addrs);
+        }
     }
 
     return answer;
@@ -680,6 +746,9 @@ struct pam_module _pam_rhosts_auth_modstruct = {
 
 /*
  * $Log: pam_rhosts_auth.c,v $
+ * Revision 1.12  1997/09/27 14:34:01  morgan
+ * fixed comment and renamed iruserok to pam_iruserok.
+ *
  * Revision 1.11  1997/04/05 06:26:39  morgan
  * fairly major fixes and enhancements (see CHANGELOG for 0.57 release)
  *
