@@ -9,6 +9,9 @@
  * This code shamelessly ripped from the pam_rootok module.
  * Slight modifications AGM. 1996/12/3
  * $Log: pam_securetty.c,v $
+ * Revision 1.7  1997/04/05 06:24:23  morgan
+ * changed return value on user unknown error
+ *
  * Revision 1.6  1997/02/15 17:30:36  morgan
  * removed fixed length syslog buffer
  *
@@ -123,7 +126,7 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 
     user_pwd = getpwnam(username);
     if (user_pwd == NULL)
-	return PAM_USER_UNKNOWN;
+	return PAM_IGNORE;
     else if (user_pwd->pw_uid != 0) /* If the user is not root,
 				       securetty's does not apply to them */
 	return PAM_SUCCESS;
