@@ -1,6 +1,6 @@
 /* pam_stress module */
 
-/* $Id: pam_stress.c,v 1.12 1997/02/15 19:06:30 morgan Exp $
+/* $Id: pam_stress.c,v 1.12 1997/02/15 19:06:30 morgan Exp morgan $
  *
  * created by Andrew Morgan <morgan@parc.power.net> 1996/3/12
  *
@@ -164,7 +164,7 @@ static int converse(pam_handle_t *pamh, int nargs
 			      , response, conv->appdata_ptr);
 	  if (retval != PAM_SUCCESS) {
 	       _pam_log(LOG_ERR,"(pam_stress) converse returned %d",retval);
-	       _pam_log(LOG_ERR,"that is: %s",pam_strerror(retval));
+	       _pam_log(LOG_ERR,"that is: %s",pam_strerror(pamh, retval));
 	  }
      } else {
 	  _pam_log(LOG_ERR,"(pam_stress) converse failed to get pam_conv");
@@ -247,6 +247,8 @@ int pam_sm_authenticate(pam_handle_t *pamh, int flags,
      char *pass;
      int ctrl;
 
+     D(("called."));
+
      ctrl = _pam_parse(argc,argv);
      _pam_report(ctrl, "pam_sm_authenticate", flags, argc, argv);
 
@@ -309,6 +311,8 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags,
 {
      int ctrl = _pam_parse(argc,argv);
 
+     D(("called. [post parsing]"));
+
      _pam_report(ctrl, "pam_sm_setcred", flags, argc, argv);
 
      if (ctrl & PAM_ST_FAIL_2)
@@ -325,6 +329,8 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 {
      int ctrl = _pam_parse(argc,argv);
 
+     D(("called. [post parsing]"));
+
      _pam_report(ctrl,"pam_sm_acct_mgmt", flags, argc, argv);
 
      if (ctrl & PAM_ST_FAIL_1)
@@ -336,7 +342,7 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 	  if (ctrl & PAM_ST_DEBUG) {
 	       _pam_log(LOG_DEBUG,"pam_sm_acct_mgmt: need a new password");
 	  }
-	  return PAM_AUTHTOKEN_REQD;
+	  return PAM_NEW_AUTHTOK_REQD;
      }
 
      return PAM_SUCCESS;
@@ -349,6 +355,8 @@ int pam_sm_open_session(pam_handle_t *pamh, int flags,
      char *username,*service;
      int ctrl = _pam_parse(argc,argv);
 
+     D(("called. [post parsing]"));
+
      _pam_report(ctrl,"pam_sm_open_session", flags, argc, argv);
 
      if ((pam_get_item(pamh, PAM_USER, (const void **) &username)
@@ -375,6 +383,8 @@ int pam_sm_close_session(pam_handle_t *pamh, int flags,
      const char *username,*service;
      int ctrl = _pam_parse(argc,argv);
 
+     D(("called. [post parsing]"));
+
      _pam_report(ctrl,"pam_sm_close_session", flags, argc, argv);
 
      if ((pam_get_item(pamh, PAM_USER, (const void **)&username)
@@ -401,6 +411,8 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
      int retval;
      int ctrl = _pam_parse(argc,argv);
 
+     D(("called. [post parsing]"));
+
      _pam_report(ctrl,"pam_sm_chauthtok", flags, argc, argv);
 
      /* this function should be called twice by the Linux-PAM library */
@@ -511,14 +523,11 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
 
 	  /* store the password */
 
-	  if (resp[0].resp && resp[1].resp) {
-	       if (strcmp(resp[0].resp,resp[1].resp)) {
+	  if (resp[i-2].resp && resp[i-1].resp) {
+	       if (strcmp(resp[i-2].resp,resp[i-1].resp)) {
 		    /* passwords are not the same; forget and return error */
 
-		    _pam_overwrite(resp[0].resp);
-		    free(resp[0].resp);
-		    _pam_overwrite(resp[1].resp);
-		    free(resp[1].resp);
+		    _pam_drop_reply(resp, i);
 
 		    if (!(flags & PAM_SILENT) && !(ctrl & PAM_ST_NO_WARN)) {
 			 pmsg[0] = &msg[0];
@@ -526,7 +535,10 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
 			 msg[0].msg = "Verification mis-typed; "
 			      "password unchaged";
 			 resp = NULL;
-			 retval = converse(pamh,1,pmsg,&resp);
+			 (void) converse(pamh,1,pmsg,&resp);
+			 if (resp) {
+			     _pam_drop_reply(resp, 1);
+			 }
 		    }
 		    return PAM_AUTHTOK_ERR;
 	       }
@@ -542,11 +554,7 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
 	       retval = PAM_SYSTEM_ERR;
 	  }
 
-	  _pam_overwrite(resp[0].resp);   /* clean up the passwords */
-	  free(resp[0].resp);
-	  _pam_overwrite(resp[1].resp);
-	  free(resp[1].resp);
-	  free(resp);
+	  _pam_drop_reply(resp, i);      /* clean up the passwords */
      } else {
 	  _pam_log(LOG_ERR,"pam_sm_chauthtok: this must be a Linux-PAM error");
 	  return PAM_SYSTEM_ERR;
